==> sil-machine-0.7.2.tar.gz: sil-machine-0.7.2/machine/translation/symmetrized_word_aligner.py <==
from typing import Iterable, Sequence, Tuple

from ..corpora.corpora_utils import batch
from .symmetrization_heuristic import SymmetrizationHeuristic
from .word_aligner import WordAligner
from .word_alignment_matrix import WordAlignmentMatrix


class SymmetrizedWordAligner(WordAligner):
    def __init__(self, src_trg_aligner: WordAligner, trg_src_aligner: WordAligner) -> None:
        self._src_trg_aligner = src_trg_aligner
        self._trg_src_aligner = trg_src_aligner
        self._heuristic = SymmetrizationHeuristic.OCH
        self.batch_size = 1024

    @property
    def heuristic(self) -> SymmetrizationHeuristic:
        return self._heuristic

    @heuristic.setter
    def heuristic(self, value: SymmetrizationHeuristic) -> None:
        self._heuristic = value

    def get_best_alignment(self, source_segment: Sequence[str], target_segment: Sequence[str]) -> WordAlignmentMatrix:
        matrix = self._src_trg_aligner.get_best_alignment(source_segment, target_segment)
        if self.heuristic is not SymmetrizationHeuristic.NONE:
            inv_matrix = self._trg_src_aligner.get_best_alignment(target_segment, source_segment)
            inv_matrix.transpose()
            matrix.symmetrize_with(inv_matrix, self.heuristic)
        return matrix

    def get_best_alignment_batch(
        self, segments: Iterable[Tuple[Sequence[str], Sequence[str]]]
    ) -> Iterable[Tuple[Sequence[str], Sequence[str], WordAlignmentMatrix]]:
        if self.heuristic is SymmetrizationHeuristic.NONE:
            yield from self._src_trg_aligner.get_best_alignment_batch(segments)
        else:
            for segments_batch in batch(segments, self.batch_size):
                results = self._src_trg_aligner.get_best_alignment_batch(segments_batch)
                inv_results = self._trg_src_aligner.get_best_alignment_batch(
                    (target_segment, source_segment) for source_segment, target_segment in segments_batch
                )
                for (source_segment, target_segment, matrix), (_, _, inv_matrix) in zip(results, inv_results):
                    inv_matrix.transpose()
                    matrix.symmetrize_with(inv_matrix, self.heuristic)
                    yield source_segment, target_segment, matrix

==> sil-machine-0.7.3.tar.gz: sil-machine-0.7.3/machine/translation/symmetrized_word_aligner.py <==
from typing import Iterable, Optional, Sequence, Tuple

from ..corpora.corpora_utils import batch
from .symmetrization_heuristic import SymmetrizationHeuristic
from .word_aligner import WordAligner
from .word_alignment_matrix import WordAlignmentMatrix


class SymmetrizedWordAligner(WordAligner):
    def __init__(self, src_trg_aligner: WordAligner, trg_src_aligner: WordAligner) -> None:
        self._src_trg_aligner = src_trg_aligner
        self._trg_src_aligner = trg_src_aligner
        self._heuristic = SymmetrizationHeuristic.OCH
        self.batch_size = 1024

    @property
    def heuristic(self) -> SymmetrizationHeuristic:
        return self._heuristic

    @heuristic.setter
    def heuristic(self, value: SymmetrizationHeuristic) -> None:
        self._heuristic = value

    def get_best_alignment(self, source_segment: Sequence[str], target_segment: Sequence[str]) -> WordAlignmentMatrix:
        matrix = self._src_trg_aligner.get_best_alignment(source_segment, target_segment)
        if self.heuristic is not SymmetrizationHeuristic.NONE:
            inv_matrix = self._trg_src_aligner.get_best_alignment(target_segment, source_segment)
            inv_matrix.transpose()
            matrix.symmetrize_with(inv_matrix, self.heuristic)
        return matrix

    def get_best_alignment_batch(
        self, segments: Iterable[Tuple[Sequence[str], Sequence[str]]], batch_size: Optional[int] = None
    ) -> Iterable[Tuple[Sequence[str], Sequence[str], WordAlignmentMatrix]]:
        if self.heuristic is SymmetrizationHeuristic.NONE:
            yield from self._src_trg_aligner.get_best_alignment_batch(segments, batch_size)
        else:
            if batch_size is None:
                batch_size = self.batch_size
            for segments_batch in batch(segments, batch_size):
                results = self._src_trg_aligner.get_best_alignment_batch(segments_batch, batch_size)
                inv_results = self._trg_src_aligner.get_best_alignment_batch(
                    ((target_segment, source_segment) for source_segment, target_segment in segments_batch), batch_size
                )
                for (source_segment, target_segment, matrix), (_, _, inv_matrix) in zip(results, inv_results):
                    inv_matrix.transpose()
                    matrix.symmetrize_with(inv_matrix, self.heuristic)
                    yield source_segment, target_segment, matrix
